/*
 * Temp8LCD.c
 *
 * Created: 2013-08-23 11:38:29
 *  Author: dsosnowski
 */ 


#define F_CPU			8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include "lcd.h"
#include "wdog.h"
#include "defs.h"
#include "ui.h"

void HardInit(void);


extern struct {
	char temp[16];
	unsigned int result[8];
	char status;
	char type;
	unsigned int temperature[8];	
}ds;

struct {
	volatile char ovf; 
	volatile char sec;
	char back;
	char idle;
	volatile char dottime;	
}tick;

struct {
	char maker;
	char level;
	char part;
	char checkpoint;
	unsigned int max[8];
	unsigned int min[8];
	char alarm[8];		
}ui;


int main(void)
{
	char button;
	
 	HardInit();
	LCDInit();
	TempInit();
	
	TempProcess();
	Intro();
	RestoreSettings();
	WdtSet(7);	
	
	tick.ovf = DOT_TIME;
	tick.dottime = 0;
	
    while(1)
    {

		//--------------------------------------
		if (tick.dottime)
		{
			tick.dottime = 0;
			if (ui.level == 0) DispBlind();
			
			LCDLocate(0,0);
			LCDData(2);
			if (CheckAlarm()) BUZZ_ON;  
		}		

		//--------------------------------------
		if (tick.sec)
		{
			tick.sec = 0;			
			BUZZ_OFF;

			//--------------------------------------
			if (TempProcess() == 0) 
			{
				TempMethod();
			}						
			if (ui.level == 0)
				DispAll();
			else
				DispPart();				

			//--------------------------------------
			if (tick.idle < IDLE_SEC) 
				tick.idle++;
			else 
			{
				tick.back = IDLE_BACK;
				ui.level = 0;
				ui.maker = 0;
				ui.part = 0;	
			}				
		}
				
		//--------------------------------------
		if (OCR2 < tick.back) OCR2++;
		if (OCR2 > tick.back) OCR2--;
		
		//--------------------------------------
		button = SwScan();
		if (button) 
		{
			tick.idle = 0;
			tick.back = NORMAL_BACK;
			OCR2 = NORMAL_BACK;
		
			RunMaker(button);
			if (ui.level == 0)
				DispAll();
			else
				DispPart();
		}		
		
		//--------------------------------------		
 		WdtReset();
		_delay_ms(20);
    }
}

//-----------------------------------------
void HardInit(void)
{	
	UI_PORT = (1<<BKL_PIN);
	UI_DDR = (1<<BKL_PIN | 1<<PIEZZO);
	
	//TCCR0 = (1<<CS02 | 1<<CS00);
	//TIMSK |= (1<<TOIE0);
	
	TCCR2 = (1<<WGM21 | 1<<WGM20 | 1<<CS22 | 1<<CS21);
	OCR2 = NORMAL_BACK;
	tick.back = NORMAL_BACK;	
	TIMSK |= (1<<TOIE2 | 1<<OCIE2);
	
	//config interrupts
	sei();		

}

//-----------------------------------------
/*ISR(TIMER0_OVF_vect)
{
	tick.ovf++;
	if (tick.ovf >= 31)
	{
		tick.ovf = 0;
		tick.sec = 1;
	} 
}*/

//-----------------------------------------
ISR(TIMER2_OVF_vect)
{
	BKL_ON;	
	tick.ovf++;
	if (tick.ovf == DOT_TIME) tick.dottime = 1;
	if (tick.ovf >= 122)
	{
		tick.ovf = 0;
		tick.sec = 1;
	} 
}

//-----------------------------------------
ISR(TIMER2_COMP_vect)
{
	BKL_OFF;	
}