/*
 * temp.c
 *
 * Created: 2012-03-06 13:21:25
 *  Author: dsosnowski
 */ 

#include <avr/io.h>
#include "dstemp.h"

//outputs for relays
#define TPORT			PORTD
#define TPIN			PIND 
#define TDDR			DDRD

//constants
#define CLEAR			0
#define STOP			0
#define DISABLE			0
#define SET				1
#define ENABLE			1

#define	TYPE_18B20		0x28
#define	TYPE_18S20		0x10


//subs


void delay_us(char us);
char onew_reset(void);
void onew_writeb(char bit);
void onew_writeB(char byte);
char onew_readb(void);
void onew_readB(char* dest);

char TempType(void);
char TempMethod(void);


struct {
	char temp[16];
	unsigned int result[8];
	char status;
	char type;
	unsigned int temperature[8];	
}ds;


//------------------------------------------
void TempInit(void)
{
	//config temp port
	TDDR = CLEAR;		//set to inputs
	TPORT = CLEAR;		//pull up off
	
	TempType();		 
}

//-------------------------------------------------------
//simple delay
void delay_us(char us)
{
  do
  {
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
  } while(--us != 0);
}

//----------------------------------
//1wire reset; 0 - ok, 1 - trouble
char onew_reset(void)
{
	char onew_trouble = 0;
			
    TDDR = 0xFF;		//set low driver			
	TPORT = 0x00;
	delay_us(250);		//wait 500us
	delay_us(250);
	
	TDDR = 0x00;		//disable low drivel
	TPORT = 0x00;
	delay_us(80);		//wait 70us
	onew_trouble |= TPIN;		//check status 0 - ok, 1 - err 
	
	delay_us(250);		//wait 500us
	delay_us(250);
	onew_trouble |= ~(TPIN);
	
	return onew_trouble;
}

//------------------------------------------
//bit write
void onew_writeb(char bit)
{
 
	TPORT = 0x00;
    TDDR = 0xFF;		//set low driver			
	delay_us(5);		//wait10us
	
	if (bit) TPORT = 0xFF;	//if bit == 1 then onew quick to H
	delay_us(65);
 
	TDDR = 0x00;		//at the end set to H
	TPORT = 0x00;
	delay_us(5);
}

//------------------------------------
//Byte write
void onew_writeB(char byte)
{
	char i;
	for (i=0; i<8; i++)
	{
		onew_writeb(byte & 0x01);
		byte >>= 1;
	}
		
}

//--------------------------------
//bits read
char onew_readb(void)
{
	char bits = 0;
	
	TPORT = 0x00;
    TDDR = 0xFF;		//set low driver			
	delay_us(5);		//wait
	TDDR = 0x00;		//set H
	
	delay_us(8);
	bits = TPIN;		//read onew state
	delay_us(57);
	return bits;
}

//---------------------------------
//Bytes read
void onew_readB(char* dest)
{
	char i;
	
	for (i=0; i<8; i++)
	{
		*dest = onew_readb();
		dest++;
	}	
}	

//-------------------------------------
//bit 0 - DS18S20; bit 1 - DS18B20
char TempType(void)
{
	char i;
	char types = 0;					//default all DS18S20
	char family;
	
	ds.status = onew_reset();
	onew_writeB(0x33);				//read ROM
	onew_readB(&ds.temp[0]);
	TempMethod();
	
	for(i=0; i<8; i++)
	{
		if (!(ds.status & (1<<i)))		//if sensor detect
		{
			family = (ds.result[i] & 0xFF);
			if (family == TYPE_18B20)
				types &= ~(1<<i);			//clr ds type flag					
			else if (family == TYPE_18S20)
				types |= (1<<i);			//set ds type flag	
			else							
				ds.status |= (1<<i);		//if unknown then set status error		
		}		
	}
	ds.type = types;
	return types;
}



//------------------------------------
char TempProcess(void)
{
	static char step = 0;
	
	if (step == 0)
	{
		ds.status = onew_reset();
		onew_writeB(0xCC);		//skip ROM
		onew_writeB(0x44);		//convert
		TPORT = 0xFF;
		TDDR = 0xFF;
		step = 1;			
	}
	else
	{
		onew_reset();
		onew_writeB(0xCC);		//skip ROM
		onew_writeB(0xBE);		//read		
		onew_readB(&ds.temp[0]);
		onew_readB(&ds.temp[8]);	
		step = 0;			
	}
	return step;
}

//-------------------------------------
char TempMethod(void)
{
	char i;
	char a;

	i = 0;
	while (i < 8)
	{
		ds.result[i] = 0;
		a = 16;
		while (a)
		{
			a--;
			ds.result[i] <<= 1;
			if (ds.temp[a] & (1<<i)) ds.result[i] += 1;							
		}
		
		TempGet(i);
		i++;
	}
	return 1;
}

//-------------------------------------
void TempGet(char channel)
{
	unsigned int buffer;
	char val;
	char decimal;
	char minus = 0;
	
	//---get coded value--------------
	buffer = ds.result[channel];
	
	//---check channel status--------
	if (ds.status & (1<<channel))	
	{	
		buffer = 0xFFFF;		//if error
	}
	else						//if ok
	{
		//---check minus------------------
		if (buffer >= 0x7FF)
		{
			buffer = ~(buffer);
			buffer += 1;
			minus = 1;
		}

		//---check type-------------------	
		if (ds.type & (1<<channel))
		{
			val = (0xFF & (buffer>>1));	
			decimal = (0x01 & buffer);	
			decimal = (decimal * 5);
		}	
		else
		{
			val = (0xFF & (buffer>>4));
			decimal = (0x0F & buffer);
			decimal = decimal * 6;
			decimal = (decimal / 10);	
		}
	
		buffer = val * 10;
		buffer = buffer + decimal;
		if (minus) 
			//buffer = buffer + 0x8000;
			buffer = 1000 - buffer;
		else
			buffer = 1000 + buffer;
	}
	ds.temperature[(7 - channel)] = buffer;
}


