/*
 * lcd.c
 *
 *  Created on: 2011-02-18
 *      Author: Damian Sosnowski
 *
 *      LCD HD44780 drivers
 */

#define F_CPU			8000000UL
#include <avr/io.h>
#include <util/delay.h>
#include "lcd.h"


//for lcd wires
#define LCD_COM_PORT		PORTB
#define LCD_COM_DDR			DDRB
#define LCD_RS				0
#define LCD_EN				1

#define LCD_DB_PORT			PORTB
#define LCD_DB_DDR			DDRB
#define LCD_DB4				2
#define LCD_DB5				3
#define LCD_DB6				4
#define LCD_DB7				5


// Komendy sterujce wywietlaczem
#define LCDC_CLS		0x01
#define LCDC_HOME		0x02
#define LCDC_MODE		0x04
	#define LCDC_MODER		0x02
	#define LCDC_MODEL		0
	#define LCDC_MODEMOVE	0x01
#define LCDC_ON 		0x08
	#define LCDC_ONDISPLAY	0x04
	#define LCDC_ONCURSOR	0x02
	#define LCDC_ONBLINK	0x01
#define LCDC_SHIFT		0x10
	#define LCDC_SHIFTDISP	0x08
	#define LCDC_SHIFTR		0x04
	#define LCDC_SHIFTL		0
#define LCDC_FUNC		0x20
	#define LCDC_FUNC8b		0x10
	#define LCDC_FUNC4b		0
	#define LCDC_FUNC2L		0x08
	#define LCDC_FUNC1L		0
	#define LCDC_FUNC5x10	0x4
	#define LCDC_FUNC5x7	0
#define LCDC_CGA		0x40
#define LCDC_DDA		0x80

#define MAX_SPEC_CHARS		8

static void LCDSendHalf(char byte);
static void LCDSendByte(char data);

//---------------------------------------------------
void LCDInit(void)
{
	//inits ports
	LCD_DB_DDR  |= (1<<LCD_DB4 | 1<<LCD_DB5 | 1<<LCD_DB6 | 1<<LCD_DB7);
	LCD_COM_DDR |= (1<<LCD_RS | 1<<LCD_EN); 
	


	_delay_ms(10);
	LCD_COM_PORT &= ~(1<<LCD_RS);
	_delay_ms(20);
	LCDSendHalf((LCDC_FUNC|LCDC_FUNC8b) >> 0x04);
	_delay_ms(10);
	LCDSendHalf((LCDC_FUNC|LCDC_FUNC8b) >> 0x04);
	_delay_ms(2);
	LCDSendHalf((LCDC_FUNC|LCDC_FUNC8b) >> 0x04);
	_delay_ms(2);
	LCDSendHalf((LCDC_FUNC|LCDC_FUNC4b) >> 0x04);
	_delay_ms(2);

	LCDCommand(LCDC_FUNC|LCDC_FUNC4b|LCDC_FUNC2L|LCDC_FUNC5x7);
	LCDCommand(LCDC_ON);
	LCDClear();
	LCDCommand(LCDC_MODE|LCDC_MODER);
	LCDCommand(LCDC_ON|LCDC_ONDISPLAY);
}

//---------------------------------------------------
//data bits mog by rozrzucone ale musz by na jednym porcie
static void LCDSendHalf(char byte)
{
	
	LCD_COM_PORT |= (1<<LCD_EN);	//set ena
	//LCD_DB_PORT |= (1<<LCD_EN);	//set ena

	//set data to port
	if (byte & (1<<3)) LCD_DB_PORT |= (1<<LCD_DB7);
	else LCD_DB_PORT &= ~(1<<LCD_DB7);
	if (byte & (1<<2)) LCD_DB_PORT |= (1<<LCD_DB6);
	else LCD_DB_PORT &= ~(1<<LCD_DB6);
	if (byte & (1<<1)) LCD_DB_PORT |= (1<<LCD_DB5);
	else LCD_DB_PORT &= ~(1<<LCD_DB5);
	if (byte & (1<<0)) LCD_DB_PORT |= (1<<LCD_DB4);
	else LCD_DB_PORT &= ~(1<<LCD_DB4);
	//_delay_us(100);

	LCD_COM_PORT &= ~(1<<LCD_EN);	//set ena
	//LCD_DB_PORT &= ~(1<<LCD_EN);	//set ena
	_delay_ms(1);
}

//---------------------------------------------------
static void LCDSendByte(char data)
{
	LCDSendHalf(data>>0x04);
	LCDSendHalf(data);
	//for the lowest possible power consumption:	

	LCD_COM_PORT |= (1<<LCD_RS | 1<<LCD_EN);	
	LCD_DB_PORT  |= (1<<LCD_DB4 | 1<<LCD_DB5 | 1<<LCD_DB6 | 1<<LCD_DB7);
	//LCD_COM_PORT |= (1<<LCD_RS);	
	//LCD_DB_PORT  |= (1<<LCD_DB4 | 1<<LCD_DB5 | 1<<LCD_DB6 | 1<<LCD_DB7 | 1<<LCD_EN);
	//DelayMs(1);
}

//---------------------------------------------------
void LCDCommand(char command)
{
	LCD_COM_PORT &= ~(1<<LCD_RS);
	LCDSendByte(command);
}

//---------------------------------------------------
void LCDData(char data)
{
	LCD_COM_PORT |= (1<<LCD_RS);
	LCDSendByte(data);
}

//---------------------------------------------------
void LCDClear(void)
{
	LCDCommand(LCDC_CLS);
	_delay_ms(2);
}

//---------------------------------------------------
void LCDHome(void)
{
	LCDCommand(LCDC_HOME);
}

//---------------------------------------------------
void LCDText(char *str)
{
	while(*str) LCDData(*str++);
}

//---------------------------------------------------
void LCDLocate(char lcd_y, char lcd_x)
{
	LCDCommand((lcd_y*0x40+lcd_x)|0x80);
}

//---------------------------------------------------
void LCDValue(unsigned int value)
{
	if(value > 999) value = 999;

	if(value > 99 ) LCDData((value/100) + 0x30);
	value %= 100;
	
	if(value >  9 ) LCDData((value/ 10) + 0x30);
	value %= 10;	
	
	LCDData(value + 0x30);
}

//---------------------------------------------------
void LCDValue_t(char value)
{
	if(value > 99) value = 99;

	if(value >  9 ) LCDData((value/ 10) + 0x30);
	else LCDData(0x30);
	
	value %= 10;	
	LCDData(value + 0x30);
}


//---------------------------------------------------
void LCDValueSpace(unsigned int value, char space)
{
	char zero_on = 0;
	if(value > 999) value = 999;
	if(value > 99)  zero_on = 1;
	if(space > 3)   space = 3;

	if(space > 2)
	{
		if(value > 99) LCDData((value/100) + 0x30);
		else           LCDData(' ');
	
	}
	
	value %= 100;
	if(space > 1)
	{
		if(value > 9)  LCDData((value/10) + 0x30);
		else           
		{
			if(zero_on) LCDData(0x30); 
			else 		LCDData(' ');
		}
	}

	value %= 10;		
	LCDData(value + 0x30);

}

//---------------------------------------------------
//if bits 7, 6, 5 sets then exit
void LCDSetSpecs(char* pData)
{
	char count = 0;
	LCDCommand(LCDC_CGA);

	while (count < MAX_SPEC_CHARS * 8)
	{
		if (*pData >= 0xE0) break;
		LCDData(*pData);
		pData++;
		count++;
	}
}

//---------------------------------------------------
void LCDSetSpecChar(char spec, char* pData)
{
	char count;
	if (spec < (MAX_SPEC_CHARS - 1))	//i.e 8-1 = 7
	{
		spec *= 8;
		LCDCommand(LCDC_CGA | spec);
		count = 0;
		while (count < 8)
		{
			LCDData(*pData);
			pData++;
			count++;
		}
	}
}

