/*
 * ui.c
 *
 * Created: 2013-08-26 10:25:23
 *  Author: dsosnowski
 */ 


#define F_CPU			8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include "lcd.h"
#include "wdog.h"
#include "defs.h"
#include "ui.h"
#include "ee_mem.h"


extern struct {
	char temp[16];
	unsigned int result[8];
	char status;
	char type;
	unsigned int temperature[8];	
}ds;

struct {
	char maker;
	char level;
	char part;
	char checkpoint;
	unsigned int max[8];
	unsigned int min[8];
	char alarm[8];	
}ui;

const char degree[8] = {0x06, 0x09, 0x09, 0x06, 0x00, 0x00, 0x00, 0x00};
const char nosmile[8] = {0x00, 0x00, 0x0A, 0x00, 0x04, 0x11, 0x0E, 0x00};
const char smile[8] = {0x00, 0x1B, 0x1B, 0x00, 0x04, 0x11, 0x0E, 0x00};

//-----------------------------------------
void DriveSettings(char button)
{
	char temp;
	if (ui.level)
	{
		temp = (ui.level - 1);
	
		//t max
		if (ui.part == 1)
		{
			if ((button == BUTTON1) || (button == HOLD1))
			{
				ui.max[temp]++;
				if (button == HOLD1) ui.max[temp]+= 20; 
				if (ui.max[temp] > 2250) ui.max[temp] = 2250;
				ui.checkpoint = SET;
			}		
	
			if ((button == BUTTON3) || (button == HOLD3))
			{
				ui.max[temp]--;
				if (button == HOLD3) ui.max[temp]-= 20; 
				if (ui.max[temp] < 500) ui.max[temp] = 500;									
				ui.checkpoint = SET;
			}		
		}
	
		//t min
		if (ui.part == 2)
		{
			if ((button == BUTTON1) || (button == HOLD1))
			{
				ui.min[temp]++;
				if (button == HOLD1) ui.min[temp]+= 20; 
				if (ui.min[temp] > 2250) ui.min[temp] = 2250;			
				ui.checkpoint = SET;
			}		
			if ((button == BUTTON3) || (button == HOLD3))
			{
				ui.min[temp]--;
				if (button == HOLD3) ui.min[temp]-= 20; 
				if (ui.min[temp] < 500) ui.min[temp] = 500;									
				ui.checkpoint = SET;
			}						
		}	
	}
}

//-----------------------------------------
void RestoreSettings(void)
{	
	char *pt;
	char index;
	
	pt = &ui.max[0];
	index = 8;
	while (index < 40)
	{
		*pt = EEread(index);
		pt++;
		index++;		
	}
	
	index = 0;
	while (index < 8)
	{
		if (ui.max[index] > 2250) ui.max[index] = 2250;
		if (ui.min[index] > 2250) ui.min[index] = 500;
		index++;		
	}		
}

//-----------------------------------------
void SaveSettings(void)
{	
	char *pt;
	char index;
	
	pt = &ui.max[0];
	index = 8;
	
	cli();
	while (index < 40)
	{
		EEwrite(index, *pt);
		pt++;
		index++;		
	}
	sei();
}

//-----------------------------------------
char CheckAlarm(void)
{	
	char index;
	char active;
	
	active = 0;
	index = 0;
	while (index < 8)
	{
		ui.alarm[index] &= ~(ALARM_ACTIVE);	
		if (ds.temperature[index] < ui.min[index]) ui.alarm[index] |= ALARM_ACTIVE;			
		if (ds.temperature[index] > ui.max[index]) ui.alarm[index] |= ALARM_ACTIVE;

		if ((ui.alarm[index] & ALARM_ACTIVE) == 0) ui.alarm[index] = 0;
		if (ds.temperature[index] == 0xFFFF) ui.alarm[index] = 0;

		if (ui.alarm[index] == 1) active = ALARM_ACTIVE;		
		index++;			
	}
	return active;
}

//-----------------------------------------
void BlockAlarm(void)
{	
	char index;
	index = 0;
	while (index < 8)
	{
		if (ui.alarm[index] & ALARM_ACTIVE) ui.alarm[index] |= ALARM_BLOCKED;
		index++;			
	}
}


//-----------------------------------------
void RunMaker(char button)
{	
	if (button == BUTTON3) 
	{
		if (ui.level == 0)
		{
			if (ui.maker < 8) ui.maker++;		
			else ui.maker = 0;			
		}
	}
	if (button == BUTTON1) 
	{
		if (ui.level == 0)
		{
			if (ui.maker) ui.maker--;
			else ui.maker = 8; 		
		}		
	}
	
	DriveSettings(button);

	
	if (button == BUTTON2) 
	{
		if (ui.level == 0)
		{
			if (ui.maker) 
			{
				ui.level = ui.maker;
				ui.maker = 0;
				ui.part = 1;
				ui.checkpoint = CLEAR;	
			}
			else
				BlockAlarm();				
		}
		else
		{
			if (ui.part == 1)
			{
				ui.maker = 0;
				ui.part = 2;				
			}
			else
			{
				//ui.maker = 0;			
				ui.maker = ui.level;			
				ui.level = 0;			
				ui.part = 0;
				
				if (ui.checkpoint == SET)
				{
					BKL_OFF;
					SaveSettings();																					
				}
			}
		}
	}
}

//-----------------------------------------
char SwScan(void)
{	
	char temp;
	static char hold;
	static char prev;
	char action = NO_SWITCH;		

	temp = ~(UI_PIN);
	temp >>= SW1_PIN;
	temp &= 0x07;
	
	if (temp == 0)
	{
		hold = 0;
		prev = 0;		
	}
	else
	{
		action = temp;
		if (hold < 200) hold++;				//counting hold time
			if (prev > 0)			//after first press - reset key			
			{
				prev = action;
				action = 0;			
			}
			else prev = action;
		if (hold >= SWITCH_THRESHOLD) //after threshold - repeat key
		{
			hold = SWITCH_REPEAT;
			action = temp;
			action |= SWITCH_HOLD;		
		}		
	}	
	return action;
}


//-----------------------------------------
void Intro(void)
{	
	LCDClear();
	LCDLocate(0,4);
	LCDData('A');
	_delay_ms(SHOW_DELAY);
	LCDData('V');
	_delay_ms(SHOW_DELAY);
	LCDData('T');
	_delay_ms(SHOW_DELAY);
	LCDData('5');
	_delay_ms(SHOW_DELAY);
	
	LCDData('4');
	_delay_ms(SHOW_DELAY);
	LCDData('8');
	_delay_ms(SHOW_DELAY);
	LCDData('9');
	WdtReset();
	//_delay_ms(SHOW_DELAY);
	_delay_ms(500);

	LCDLocate(1,0);
	LCDText("www.sklep.avt.pl");
	_delay_ms(500);
	_delay_ms(500);
	WdtReset();

	LCDSetSpecChar(0, degree);
	LCDSetSpecChar(1, nosmile);
	LCDSetSpecChar(2, smile);

}

//-----------------------------------------
void TempDisp(unsigned int temp)
{
	char digit;
	char fillsecond = 0;
	
	if (temp < 0xFFFF)
	{
		if (temp < 1000)
		{
			LCDData('-');
			temp = (1000 - temp);			

			digit = (temp / 1000);
			temp %= 1000;
		}
		else
		{
			temp = (temp - 1000);			
			digit = (temp / 1000);
			temp %= 1000;
		
			if (digit) 
			{
				LCDData(digit + 0x30);
				fillsecond = 1;
			}
			else       
				LCDData(' ');		
		}
		
		digit = (temp / 100);
		temp %= 100;
		if (digit) LCDData(digit + 0x30);
		else       
		{
			if (fillsecond) 
				LCDData(0x30);
			else 
				LCDData(' ');
		}
		
		digit = (temp / 10);
		temp %= 10;
		LCDData(digit + 0x30);

		LCDData('.');
	
		digit = (temp & 0xFF);
		LCDData(digit + 0x30);
		
		LCDData(0); 
		LCDData('C');
	}
	else
		LCDText("       ");

}

//-----------------------------------------
void DispAll(void)
{	
	LCDLocate(0,0);    
	if (ui.maker == 1) LCDData('>'); 
	//else LCDData(' ');
	else LCDData(1);
	TempDisp(ds.temperature[0]);
	if (ui.maker == 5) LCDData('>'); 
	else LCDData(' ');	
	TempDisp(ds.temperature[4]);
	
	LCDLocate(1,0);
	if (ui.maker == 2) LCDData('>'); 
	else LCDData(' ');
	TempDisp(ds.temperature[1]);
	if (ui.maker == 6) LCDData('>'); 
	else LCDData(' ');	
	TempDisp(ds.temperature[5]);

	LCDLocate(0,16);
	if (ui.maker == 3) LCDData('>'); 
	else LCDData(' ');
	TempDisp(ds.temperature[2]);
	if (ui.maker == 7) LCDData('>'); 
	else LCDData(' ');	
	TempDisp(ds.temperature[6]);

	LCDLocate(1,16);
	if (ui.maker == 4) LCDData('>'); 
	else LCDData(' ');
	TempDisp(ds.temperature[3]);
	if (ui.maker == 8) LCDData('>'); 
	else LCDData(' ');	
	TempDisp(ds.temperature[7]);
}

//-----------------------------------------
void DispPart(void)
{	
	char temp = (ui.level - 1);
	
	if (ui.level <= 4)
	{
		LCDLocate(0,0);    
		if (ui.level == 1)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;

		LCDLocate(1,0);    
		if (ui.level == 2)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;
		LCDLocate(0,16);    
		if (ui.level == 3)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;
		LCDLocate(1,16);    
		if (ui.level == 4)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;

		LCDLocate(0,8);    
		LCDText("   Max  ");						
			LCDLocate(1,8);    
			if (ui.part == 1) LCDData('>'); 
			else LCDData(' ');
			TempDisp(ui.max[temp]);

		LCDLocate(0,24);    
		LCDText("   Min  ");						
			LCDLocate(1,24);    
			if (ui.part == 2) LCDData('>'); 
			else LCDData(' ');
			TempDisp(ui.min[temp]);
	}
	else
	{
		LCDLocate(0,8);    
		if (ui.level == 5)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;

		LCDLocate(1,8);    
		if (ui.level == 6)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;
		LCDLocate(0,24);    
		if (ui.level == 7)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;
		LCDLocate(1,24);    
		if (ui.level == 8)
		{
			LCDData(' ');
			TempDisp(ds.temperature[temp]);
		}
		else
			LCDText("        ") ;
			
		LCDLocate(0,0);    
		LCDText("   Max  ");						
			LCDLocate(1,0);    
			if (ui.part == 1) LCDData('>'); 
			else LCDData(' ');
			TempDisp(ui.max[temp]);

		LCDLocate(0,16);    
		LCDText("   Min  ");						
			LCDLocate(1,16);    
			if (ui.part == 2) LCDData('>'); 
			else LCDData(' ');
			TempDisp(ui.min[temp]);
	}
}

//-----------------------------------------
void DispBlind(void)
{	
	char index;
	
	index = 0;
	while (index < 8)
	{
		if (ui.alarm[index]) 		
		{
			if (index == 0) LCDLocate(0,0);    
			if (index == 1) LCDLocate(1,0);    
			if (index == 2) LCDLocate(0,16);    
			if (index == 3) LCDLocate(1,16);    
			if (index == 4) LCDLocate(0,8);    
			if (index == 5) LCDLocate(1,8);    
			if (index == 6) LCDLocate(0,24);    
			if (index == 7) LCDLocate(1,24);    
			LCDText("        ");	
		}
		index++;		
	}			
}